// -------- CODE FOR DRAG & DROP ----------------------

int x, y, difX, difY, count;
ArrayList<PVector> towers; // Towers that are placed down

boolean within; // If mouse was held down during the previous frame
final int towerSize = 25;

//*****EDIT HERE: use your color of choice for your toewr here!*****
final color towerColour = #7b9d32;
//these variables are the trash bin coordinates
int trashX1, trashY1, trashX2, trashY2;
void setup() {
  size(800, 500);
  initDragAndDrop();
  initPath();
}
void draw() {
  background(#add558);
  drawPath();

  drawAllTowers();
  drawTrash();
  drawSelectedTowers();
  dragAndDropInstructions();
}
void mouseDragged() {
  if (within) { // Check to see if the user is currently dragging a tower
    // Set the values while accounting for the offset
    x = mouseX + difX;
    y = mouseY + difY;
  }
}
// Whenever the user initially presses down on the mouse
void mousePressed() {
  within = withinBounds(); // Check to see if the pointer is within the bounds of the tower

  if (within) {
    handlePickUp(); // The tower has been "picked up"
    difX = x - mouseX; // Calculate the offset values (the mouse pointer may not be in the direct centre of the tower)
    difY = y - mouseY;
  }
}
// Whenever the user releases their mouse
void mouseReleased() {
  if (within) { // If the user was holding the tower in the previous frame, the tower has just been dropped
    handleDrop(); // Call the method to handle the drop and check for drop validity
  }

  within = false; // The mouse is no longer holding the tower
}
void initDragAndDrop() {
  x = 650;
  y = 50;

  within = false;
  difX = 0;
  difY = 0;

  trashX1 = 525;
  trashY1 = 30;
  trashX2 = 775;
  trashY2 = 120;

  count = 0;
  towers = new ArrayList<PVector>();
}




//*****EDIT ZONE STARTS*****

//*****EDIT HERE: write the point rectangle collision here!*****
boolean pointRectCollision(float x1, float y1, float x2, float y2, float size) {
  
}

//*****EDIT HERE: write the trash collision check here!*****
boolean trashDrop() {
  
}

//*****EDIT HERE: write the legal drop checking here
boolean legalDrop() {
}

//*****EDIT ZONE ENDS*****



// Check to see if mouse pointer is within the boundaries of the tower
boolean withinBounds() {
  return pointRectCollision(mouseX, mouseY, x, y, towerSize);
}

// -------Methods Used for further interaction-------
void handleDrop() { // Will be called whenever a tower is placed down
  // Instructions to check for valid drop area will go here
  if (trashDrop()) {
    x = 650;
    y = 50;
    println("Dropped object in trash.");
  } else if (legalDrop()) {
    towers.add(new PVector(x, y));
    // Add the tower to the list of placed down towers
    x = 650;
    y = 50;
    println("Dropped for the " + (++count) + "th time.");
  }
}

// Will be called whenever a tower is picked up
void handlePickUp() {
  println("Object picked up.");
}
// --------------------------------------------------

// Draw a simple tower at a specified location
void drawTowerIcon(float xPos, float yPos, color colour) {
  strokeWeight(0);
  stroke(0);
  fill(colour);
  rectMode(CENTER);
  rect(xPos, yPos, towerSize, towerSize); // Draw a simple rectangle as the tower
}
// Draws a tower that rotates to face the targetLocation
void drawTowerIcon(float xPos, float yPos, color colour, PVector targetLocation) {
  strokeWeight(5);
  stroke(#4C6710);

  line(xPos, yPos, targetLocation.x, targetLocation.y);
  pushMatrix();
  translate(xPos, yPos);

  // Angle calculation

  float slope = (targetLocation.y - yPos) / (targetLocation.x - xPos);
  float angle = atan(slope);


  rotate(angle);

  strokeWeight(0);
  fill(colour);
  rectMode(CENTER);
  rect(0, 0, towerSize, towerSize); // Draw a simple rectangle as the tower

  popMatrix();
}
void drawAllTowers() {
  for (int i = 0; i < towers.size(); i++) {
    float xPos = towers.get(i).x, yPos = towers.get(i).y;
    drawTowerIcon(xPos, yPos, towerColour, new PVector(mouseX, mouseY)); // Towers will track the mouse as a placeholder
    fill(#4C6710);
    text("Tower " + (i+1), xPos - 30, yPos - 20);
  }
}
void drawSelectedTowers() {
  // Draws the tower in the top right and the tower you drag
  // Changing the color if it is an illegal drop to red
  if (!legalDrop()) {
    drawTowerIcon(x, y, #FF0000); // Draw the current tower (that the user is holding) as red to indicate illegal
  } else {
    drawTowerIcon(x, y, towerColour); // Draw the current tower (that the user is holding)
  }

  drawTowerIcon(650, 50, towerColour); // Draw the pick-up tower on the top right
}

void drawTrash() {
  rectMode(CORNERS);
  noStroke();
  fill(#4C6710);
  rect(trashX1, trashY1, trashX2, trashY2);
  fill(255, 255, 255);
  stroke(255, 255, 255);
}

void dragAndDropInstructions() {
  fill(#4C6710);
  textSize(12);

  text("Pick up tower from here!", 620, 20);
  text("You can't place towers on the path of the balloons!", 200, 20);
  text("Place a tower into the surrounding area to put it in the trash.", 200, 40);
  text("Mouse X: " + mouseX + "\nMouse Y: " + mouseY + "\nMouse held: " + mousePressed + "\nWithin object bounds: " + within, 15, 20);
}


// -------- CODE FOR PATH COLLISION DETECTION ---------
float pointDistToLine(PVector start, PVector end, PVector point) {
  // Code from https://stackoverflow.com/questions/849211/shortest-distance-between-a-point-and-a-line-segment
  float l2 = (start.x - end.x) * (start.x - end.x) + (start.y - end.y) * (start.y - end.y);  // i.e. |w-v|^2 -  avoid a sqrt
  if (l2 == 0.0) return dist(end.x, end.y, point.x, point.y);   // v == w case
  float t = max(0, min(1, PVector.sub(point, start).dot(PVector.sub(end, start)) / l2));
  PVector projection = PVector.add(start, PVector.mult(PVector.sub(end, start), t));  // Projection falls on the segment
  return dist(point.x, point.y, projection.x, projection.y);
}

float shortestDist(PVector point) {
  float answer = Float.MAX_VALUE;
  for (int i = 0; i < points.size() - 1; i++) {
    PVector start = points.get(i);
    PVector end = points.get(i + 1);
    float distance = pointDistToLine(start, end, point);
    answer = min(answer, distance);
  }
  return answer;
}
